@@ -100,7 +100,7 @@ gs_listener_vtable = { &gs_listener_unregister_handler,
 static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE (GSListener, gs_listener, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GSListener, gs_listener, G_TYPE_OBJECT)
 
 GQuark
 gs_listener_error_quark (void)
@@ -118,6 +118,31 @@ gs_listener_unregister_handler (DBusConnection *connection,
 {
 }
 
+static gboolean
+send_dbus_message (DBusConnection *connection,
+                   DBusMessage    *message)
+{
+        gboolean is_connected;
+        gboolean sent;
+
+        g_return_val_if_fail (message != NULL, FALSE);
+
+        if (! connection) {
+                g_warning ("There is no valid connection to the message bus");
+                return FALSE;
+        }
+
+        is_connected = dbus_connection_get_is_connected (connection);
+        if (! is_connected) {
+                g_warning ("Not connected to the message bus");
+                return FALSE;
+        }
+
+        sent = dbus_connection_send (connection, message, NULL);
+
+        return sent;
+}
+
 static void
 gs_listener_send_signal_active_changed (GSListener *listener)
 {
@@ -135,7 +160,7 @@ gs_listener_send_signal_active_changed (GSListener *listener)
         dbus_message_iter_init_append (message, &iter);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &active);
 
-        if (! dbus_connection_send (listener->priv->connection, message, NULL)) {
+        if (! send_dbus_message (listener->priv->connection, message)) {
                 g_warning ("Could not send ActiveChanged signal");
         }
 
@@ -159,7 +184,7 @@ gs_listener_send_signal_throttle_enabled_changed (GSListener *listener)
         dbus_message_iter_init_append (message, &iter);
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &enabled);
 
-        if (! dbus_connection_send (listener->priv->connection, message, NULL)) {
+        if (! send_dbus_message (listener->priv->connection, message)) {
                 g_warning ("Could not send ThrottleEnabledChanged signal");
         }
 
@@ -287,24 +312,28 @@ listener_property_set_bool (GSListener *listener,
                             guint       prop_id,
                             dbus_bool_t value)
 {
+        dbus_bool_t ret;
+
+        ret = FALSE;
+
         switch (prop_id) {
         case PROP_ACTIVE:
                 gs_listener_set_active (listener, value);
-                return TRUE;
+                ret = TRUE;
                 break;
         case PROP_IDLE:
                 gs_listener_set_idle (listener, value);
-                return TRUE;
+                ret = TRUE;
                 break;
         case PROP_THROTTLE_ENABLED:
                 gs_listener_set_throttle_enabled (listener, value);
-                return TRUE;
+                ret = TRUE;
                 break;
         default:
                 break;
         }
 
-        return FALSE;
+        return ret;
 }
 
 static void
@@ -347,14 +376,11 @@ listener_add_inhibitor (GSListener     *listener,
                         DBusConnection *connection,
                         DBusMessage    *message)
 {
-        const char     *path;
         const char     *sender;
         DBusMessage    *reply;
         DBusError       error;
         char           *reason;
 
-        path = dbus_message_get_path (message);
-
         dbus_error_init (&error);
         if (! dbus_message_get_args (message, &error,
                                      DBUS_TYPE_STRING, &reason,
@@ -393,13 +419,10 @@ listener_remove_inhibitor (GSListener     *listener,
                            DBusConnection *connection,
                            DBusMessage    *message)
 {
-        const char  *path;
         DBusMessage *reply;
         DBusError    error;
         const char  *sender;
 
-        path = dbus_message_get_path (message);
-
         dbus_error_init (&error);
         if (! dbus_message_get_args (message, &error,
                                      DBUS_TYPE_INVALID)) {
@@ -472,7 +495,7 @@ raise_property_type_error (DBusConnection *connection,
                                         buf);
         if (reply == NULL)
                 g_error ("No memory");
-        if (!dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL))
                 g_error ("No memory");
 
         dbus_message_unref (reply);
@@ -519,7 +542,7 @@ listener_set_property (GSListener     *listener,
         if (reply == NULL)
                 g_error ("No memory");
 
-        if (!dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL))
                 g_error ("No memory");
 
         dbus_message_unref (reply);
@@ -533,12 +556,9 @@ listener_get_property (GSListener     *listener,
                        DBusMessage    *message,
                        guint           prop_id)
 {
-        const char     *path;
         DBusMessageIter iter;
         DBusMessage    *reply;
 
-        path = dbus_message_get_path (message);
-
         reply = dbus_message_new_method_return (message);
 
         dbus_message_iter_init_append (reply, &iter);
@@ -569,11 +589,11 @@ listener_get_property (GSListener     *listener,
                 }
                 break;
         default:
-                g_warning ("Unsupported property id %d", prop_id);
+                g_warning ("Unsupported property id %u", prop_id);
                 break;
         }
 
-        if (!dbus_connection_send (connection, reply, NULL))
+        if (! dbus_connection_send (connection, reply, NULL))
                 g_error ("No memory");
 
         dbus_message_unref (reply);
@@ -586,13 +606,10 @@ listener_get_idle_time (GSListener     *listener,
                         DBusConnection *connection,
                         DBusMessage    *message)
 {
-        const char     *path;
         DBusMessageIter iter;
         DBusMessage    *reply;
         dbus_uint32_t    secs;
 
-        path = dbus_message_get_path (message);
-
         reply = dbus_message_new_method_return (message);
 
         dbus_message_iter_init_append (reply, &iter);
@@ -702,7 +719,7 @@ gs_listener_message_handler (DBusConnection *connection,
                 if (reply == NULL)
                         g_error ("No memory");
 
-                if (!dbus_connection_send (connection, reply, NULL))
+                if (! dbus_connection_send (connection, reply, NULL))
                         g_error ("No memory");
 
                 dbus_message_unref (reply);
@@ -718,6 +735,48 @@ gs_listener_message_handler (DBusConnection *connection,
         else return listener_dbus_filter_handle_methods (connection, message, user_data, TRUE);
 }
 
+static gboolean
+gs_listener_dbus_init (GSListener *listener)
+{
+        DBusError error;
+
+        dbus_error_init (&error);
+
+        listener->priv->connection = dbus_bus_get (DBUS_BUS_SESSION, &error);
+
+        if (listener->priv->connection == NULL) {
+                if (dbus_error_is_set (&error)) {
+                        g_warning ("couldn't connect to session bus: %s",
+                                   error.message);
+                        dbus_error_free (&error);
+                }
+                return FALSE;
+        }
+
+        dbus_connection_setup_with_g_main (listener->priv->connection, NULL);
+	dbus_connection_set_exit_on_disconnect (listener->priv->connection, FALSE);
+
+        return TRUE;
+}
+
+static gboolean
+reinit_dbus (GSListener *listener)
+{
+        gboolean initialized;
+        gboolean try_again;
+
+        initialized = gs_listener_dbus_init (listener);
+
+        /* if we didn't initialize then try again */
+        /* FIXME: Should we keep trying forever?  If we fail more than
+           once or twice then the session bus may have died.  The
+           problem is that if it is restarted it will likely have a
+           different bus address and we won't be able to find it */
+        try_again = !initialized;
+
+        return try_again;
+}
+
 static DBusHandlerResult
 listener_dbus_filter_function (DBusConnection *connection,
                                DBusMessage    *message,
@@ -738,17 +797,13 @@ listener_dbus_filter_function (DBusConnection *connection,
         if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected") &&
             strcmp (path, DBUS_PATH_LOCAL) == 0) {
 
-                /* this is a local message; e.g. from libdbus in this process */
-
                 g_message ("Got disconnected from the system message bus; "
-                           "retrying to reconnect every 3000 ms");
+                           "retrying to reconnect every 10 seconds");
 
-#if 0
-                dbus_connection_unref (dbus_connection);
-                dbus_connection = NULL;
+                dbus_connection_unref (connection);
+                connection = NULL;
 
-                g_timeout_add (3000, reinit_dbus, NULL);
-#endif
+                g_timeout_add (10000, (GSourceFunc)reinit_dbus, listener);
         } else if (dbus_message_is_signal (message,
                                            DBUS_INTERFACE_DBUS,
                                            "NameOwnerChanged")) {
@@ -909,9 +964,9 @@ gs_listener_class_init (GSListenerClass *klass)
 static gboolean
 screensaver_is_running (DBusConnection *connection)
 {
-        DBusError               error;
-        gboolean                exists;
-
+        DBusError error;
+        gboolean  exists;
+              
         g_return_val_if_fail (connection != NULL, FALSE);
 
         dbus_error_init (&error);
@@ -926,12 +981,13 @@ gboolean
 gs_listener_acquire (GSListener *listener,
                      GError    **error)
 {
-        gboolean acquired;
+        gboolean  acquired;
         DBusError buserror;
+        gboolean  is_connected;
 
         g_return_val_if_fail (listener != NULL, FALSE);
 
-        if (!listener->priv->connection) {
+        if (! listener->priv->connection) {
                 g_set_error (error,
                              GS_LISTENER_ERROR,
                              GS_LISTENER_ERROR_ACQUISITION_FAILURE,
@@ -940,6 +996,16 @@ gs_listener_acquire (GSListener *listener,
                 return FALSE;
         }
 
+        is_connected = dbus_connection_get_is_connected (listener->priv->connection);
+        if (! is_connected) {
+                g_set_error (error,
+                             GS_LISTENER_ERROR,
+                             GS_LISTENER_ERROR_ACQUISITION_FAILURE,
+                             "%s",
+                             _("not connected to the message bus"));
+                return FALSE;
+        }
+
         if (screensaver_is_running (listener->priv->connection)) {
                 g_set_error (error,
                              GS_LISTENER_ERROR,
@@ -954,8 +1020,10 @@ gs_listener_acquire (GSListener *listener,
         if (dbus_connection_register_object_path (listener->priv->connection,
                                                   GS_LISTENER_PATH,
                                                   &gs_listener_vtable,
-                                                  listener) == FALSE)
+                                                  listener) == FALSE) {
                 g_critical ("out of memory registering object path");
+                return FALSE;
+        }
 
         acquired = dbus_bus_request_name (listener->priv->connection,
                                           GS_LISTENER_SERVICE,
@@ -984,20 +1052,9 @@ gs_listener_acquire (GSListener *listener,
 static void
 gs_listener_init (GSListener *listener)
 {
-        DBusError error;
-
         listener->priv = GS_LISTENER_GET_PRIVATE (listener);
 
-        dbus_error_init (&error);
-        listener->priv->connection = dbus_bus_get (DBUS_BUS_SESSION, &error);
-        if (listener->priv->connection == NULL) {
-                g_critical ("couldn't connect to session bus: %s",
-                            error.message);
-        } else {
-                dbus_connection_setup_with_g_main (listener->priv->connection, NULL);
-        }
-
-        dbus_error_free (&error);
+        gs_listener_dbus_init (listener);
 }
 
 static void
@@ -1012,7 +1069,8 @@ gs_listener_finalize (GObject *object)
 
         g_return_if_fail (listener->priv != NULL);
 
-        /*dbus_connection_unref (listener->priv->connection);*/
+        if (listener->priv->connection)
+                dbus_connection_unref (listener->priv->connection);
 
         if (listener->priv->inhibitors)
                 g_hash_table_destroy (listener->priv->inhibitors);
