@@ -59,7 +59,7 @@ enum {
 
 static GObjectClass *parent_class = NULL;
 
-G_DEFINE_TYPE (GSMonitor, gs_monitor, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GSMonitor, gs_monitor, G_TYPE_OBJECT)
 
 static void
 gs_monitor_class_init (GSMonitorClass *klass)
@@ -80,7 +80,7 @@ manager_activated_cb (GSManager *manager,
 }
 
 static void
-manager_disactivated_cb (GSManager *manager,
+manager_deactivated_cb (GSManager *manager,
                          GSMonitor *monitor)
 {
         gs_listener_set_active (monitor->priv->listener, FALSE);
@@ -138,7 +138,7 @@ listener_active_changed_cb (GSListener *listener,
                 /* turn off the idleness watcher */
                 res = gs_watcher_set_active (monitor->priv->watcher, FALSE);
                 if (! res) {
-                        g_warning ("Unable to disactivate the idle watcher");
+                        g_warning ("Unable to deactivate the idle watcher");
                         return FALSE;
                 }
 
@@ -180,7 +180,7 @@ listener_active_changed_cb (GSListener *listener,
                 /* disable power management */
                 res = gs_power_set_active (monitor->priv->power, FALSE);
                 if (! res) {
-                        g_warning ("Unable to disactivate power management");
+                        g_warning ("Unable to deactivate power management");
                 }
         }
 
@@ -245,11 +245,10 @@ _gs_monitor_update_from_prefs (GSMonitor *monitor,
         gs_manager_set_mode (monitor->priv->manager, monitor->priv->prefs->mode);
         gs_manager_set_themes (monitor->priv->manager, monitor->priv->prefs->themes);
 
+        gs_watcher_set_timeout (monitor->priv->watcher, monitor->priv->prefs->timeout);
         gs_watcher_set_active (monitor->priv->watcher,
                                monitor->priv->prefs->mode != GS_MODE_DONT_BLANK);
 
-        gs_watcher_set_timeout (monitor->priv->watcher, monitor->priv->prefs->timeout);
-
         gs_power_set_timeouts (monitor->priv->power,
                                monitor->priv->prefs->dpms_standby,
                                monitor->priv->prefs->dpms_suspend,
@@ -289,8 +288,8 @@ gs_monitor_init (GSMonitor *monitor)
         monitor->priv->manager = gs_manager_new ();
         g_signal_connect (monitor->priv->manager, "activated",
                           G_CALLBACK (manager_activated_cb), monitor);
-        g_signal_connect (monitor->priv->manager, "disactivated",
-                          G_CALLBACK (manager_disactivated_cb), monitor);
+        g_signal_connect (monitor->priv->manager, "deactivated",
+                          G_CALLBACK (manager_deactivated_cb), monitor);
 
         monitor->priv->power = gs_power_new ();
         g_signal_connect (monitor->priv->power, "changed",
